@@ -119,6 +119,38 @@ public class DFABuilder implements NinaAction {
 
 	}
 
+	private static class DG {
+
+		private int num;
+		private boolean sign;
+
+		DG(int n, boolean s) {
+			num = n;
+			sign = s;
+		}
+
+	}
+
+	private static class DH {
+
+		private int num;
+
+		DH(int n) {
+			num = n;
+		}
+
+	}
+
+	private static class DO {
+
+		private int num;
+
+		DO(int n) {
+			num = n;
+		}
+
+	}
+
 	/**
 	 * 
 	 *
@@ -516,6 +548,28 @@ public class DFABuilder implements NinaAction {
 
 	//
 	private static final Object _END = new Object();
+	private static final Object _FLOAT = new Object();
+
+	//
+	private static final Range DIGIT =
+			IntervalsInt.newClosedInterval('0', '9');
+	private static final Range SIGN_P =
+			IntervalsInt.newClosedInterval('+', '+');
+	private static final Range SIGN_N =
+			IntervalsInt.newClosedInterval('-', '-');
+	private static final Range HEXD1 =
+			IntervalsInt.newClosedInterval('A', 'F');
+	private static final Range HEXD2 =
+			IntervalsInt.newClosedInterval('a', 'f');
+	private static final Range HEXD =
+			DIGIT.join(HEXD1).join(HEXD2);
+	private static final Range OCTD =
+			IntervalsInt.newClosedInterval('0', '7');
+	private static final Range POINT =
+			IntervalsInt.newClosedInterval('.', '.');
+	private static final Range CHAR_E =
+			IntervalsInt.newClosedInterval('e', 'e').join(
+					IntervalsInt.newClosedInterval('E', 'E'));
 
 	//
 	private Map<String, DBS> map = new HashMap<String, DBS>();
@@ -806,6 +860,199 @@ public class DFABuilder implements NinaAction {
 		}
 	}
 
+	private void _linkDigitsBig(NinaEvent q, DBS v, boolean s) {
+		NinaLanguageOption p = q.getLanguageOption();
+		String m, ib, ab, ed;
+		DBS x, y = null;
+
+		m  = mealy != null ? mealy.toString() : "";
+		ib = p.getInitBuffer(p.getSequenceBufferName());
+		ab = p.getAppendBuffer(p.getSequenceBufferName());
+		ed = p.getUnget() + p.getBufferToBigInteger(10) + m;
+
+		if(s) {
+			y = createState(q);
+			v.linkAlphabet(y, SIGN_P, ib);
+			v.linkAlphabet(y, SIGN_N, ib + ab);
+		}
+
+		x = createState(q);
+		v.linkAlphabet(x, DIGIT, ib + ab);
+		if(y != null) {
+			y.linkAlphabet(x, DIGIT, ab);
+		}
+		x.linkAlphabet(x, DIGIT, ab);
+		x.linkOthers(vertex, ed);
+		x.linkEnd(vertex, ed);
+	}
+
+	private void _linkDigitsInt(NinaEvent q, DBS v, int d, boolean s) {
+		NinaLanguageOption p = q.getLanguageOption();
+		String m, ib, ab, ed;
+		DBS x, y = v;
+
+		m  = mealy != null ? mealy.toString() : "";
+		ib = p.getInitBuffer(p.getSequenceBufferName());
+		ab = p.getAppendBuffer(p.getSequenceBufferName());
+		ed = p.getBufferToInteger(10) + m;
+
+		if(s) {
+			y = createState(q);
+			v.linkAlphabet(y, SIGN_P, ib);
+			v.linkAlphabet(y, SIGN_N, ib + ab);
+			v.linkOthers(y, ib);
+			ib = "";
+		}
+
+		if(d > 1) {
+			x = createState(q);
+			y.linkAlphabet(x, DIGIT, ib + ab);
+			for(int i = 1; i < d - 1; i++) {
+				y = x;  x = createState(q);
+				y.linkAlphabet(x, DIGIT, ab);
+				y.linkOthers(vertex, p.getUnget() + ed);
+				y.linkEnd(vertex, p.getUnget() + ed);
+			}
+			x.linkAlphabet(vertex, DIGIT, ab + ed);
+			x.linkOthers(vertex, p.getUnget() + ed);
+			x.linkEnd(vertex, p.getUnget() + ed);
+		} else {
+			y.linkAlphabet(vertex, DIGIT, ib + ab + ed);
+		}
+	}
+
+	private void _linkHexBig(NinaEvent q, DBS v) {
+		NinaLanguageOption p = q.getLanguageOption();
+		String m, ib, ab, ed;
+		DBS x;
+
+		m  = mealy != null ? mealy.toString() : "";
+		ib = p.getInitBuffer(p.getSequenceBufferName());
+		ab = p.getAppendBuffer(p.getSequenceBufferName());
+		ed = p.getUnget() + p.getBufferToBigInteger(16) + m;
+		x  = createState(q);
+		v.linkAlphabet(x, HEXD, ib + ab);
+		x.linkAlphabet(x, HEXD, ab);
+		x.linkOthers(vertex, ed);
+		x.linkEnd(vertex, ed);
+	}
+
+	private void _linkHexInt(NinaEvent q, DBS v, int d) {
+		NinaLanguageOption p = q.getLanguageOption();
+		String m, ib, ab, ed;
+		DBS x, y = v;
+
+		m  = mealy != null ? mealy.toString() : "";
+		ib = p.getInitBuffer(p.getSequenceBufferName());
+		ab = p.getAppendBuffer(p.getSequenceBufferName());
+		ed = p.getBufferToInteger(16) + m;
+
+		x = createState(q);
+		y.linkAlphabet(x, HEXD, ib + ab);
+		for(int i = 1; i < d / 4 - 1; i++) {
+			y = x;  x = createState(q);
+			y.linkAlphabet(x, HEXD, ab);
+			y.linkOthers(vertex, p.getUnget() + ed);
+			y.linkEnd(vertex, p.getUnget() + ed);
+		}
+		x.linkAlphabet(vertex, HEXD, ab + ed);
+		x.linkOthers(vertex, p.getUnget() + ed);
+		x.linkEnd(vertex, p.getUnget() + ed);
+	}
+
+	private void _linkOctBig(NinaEvent q, DBS v) {
+		NinaLanguageOption p = q.getLanguageOption();
+		String m, ib, ab, ed;
+		DBS x;
+
+		m  = mealy != null ? mealy.toString() : "";
+		ib = p.getInitBuffer(p.getSequenceBufferName());
+		ab = p.getAppendBuffer(p.getSequenceBufferName());
+		ed = p.getUnget() + p.getBufferToBigInteger(8) + m;
+		x  = createState(q);
+		v.linkAlphabet(x, OCTD, ib + ab);
+		x.linkAlphabet(x, OCTD, ab);
+		x.linkOthers(vertex, ed);
+		x.linkEnd(vertex, ed);
+	}
+
+	private void _linkOctInt(NinaEvent q, DBS v, int d) {
+		NinaLanguageOption p = q.getLanguageOption();
+		String m, ib, ab, ed;
+		DBS x, y = v;
+
+		m  = mealy != null ? mealy.toString() : "";
+		ib = p.getInitBuffer(p.getSequenceBufferName());
+		ab = p.getAppendBuffer(p.getSequenceBufferName());
+		ed = p.getBufferToInteger(8) + m;
+
+		x = createState(q);
+		v.linkAlphabet(x,
+				IntervalsInt.newClosedInterval(
+						'0', '0' + (1 << (d % 3)) - 1),
+				ib + ab);
+		y = x;  x = createState(q);
+		v.linkAlphabet(x,
+				IntervalsInt.newClosedInterval(
+						'0' + (1 << (d % 3)), '7'),
+				ib + ab);
+		y.linkAlphabet(x, OCTD, ab);
+		y.linkOthers(vertex, p.getUnget() + ed);
+		y.linkEnd(vertex, p.getUnget() + ed);
+		for(int i = 2; i < d / 3; i++) {
+			y = x;  x = createState(q);
+			y.linkAlphabet(x, OCTD, ab);
+			y.linkOthers(vertex, p.getUnget() + ed);
+			y.linkEnd(vertex, p.getUnget() + ed);
+		}
+		x.linkAlphabet(vertex, OCTD, ab + ed);
+		x.linkOthers(vertex, p.getUnget() + ed);
+		x.linkEnd(vertex, p.getUnget() + ed);
+	}
+
+	private void _linkFloat(NinaEvent q, DBS v) {
+		NinaLanguageOption p = q.getLanguageOption();
+		String m, ib, ab, e1, e2;
+		DBS sint, ssgn, sfpt, sdec, sep1, sep2, sepd;
+
+		m  = mealy != null ? mealy.toString() : "";
+		ib = p.getInitBuffer(p.getSequenceBufferName());
+		ab = p.getAppendBuffer(p.getSequenceBufferName());
+		e1 = p.getUnget() + p.getBufferToNumberInteger() + m;
+		e2 = p.getUnget() + p.getBufferToNumberDouble() + m;
+
+		ssgn = createState(q);
+		v.linkAlphabet(ssgn, SIGN_P, ib);
+		v.linkAlphabet(ssgn, SIGN_N, ib + ab);
+		sint = createState(q);
+		v.linkAlphabet(sint, DIGIT, ib + ab);
+		ssgn.linkAlphabet(sint, DIGIT, ab);
+		sint.linkAlphabet(sint, DIGIT, ab);
+		sint.linkOthers(vertex, e1);
+		sint.linkEnd(vertex, e1);
+		sfpt = createState(q);
+		v.linkAlphabet(sfpt, POINT, ib + ab);
+		ssgn.linkAlphabet(sfpt, POINT, ab);
+		sint.linkAlphabet(sfpt, POINT, ab);
+		sdec = createState(q);
+		sfpt.linkAlphabet(sdec, DIGIT, ab);
+		sdec.linkAlphabet(sdec, DIGIT, ab);
+		sdec.linkOthers(vertex, e2);
+		sdec.linkEnd(vertex, e2);
+		sep1 = createState(q);
+		sint.linkAlphabet(sep1, CHAR_E, ab);
+		sdec.linkAlphabet(sep1, CHAR_E, ab);
+		sep2 = createState(q);
+		sep1.linkAlphabet(sep2, SIGN_P, ab);
+		sep1.linkAlphabet(sep2, SIGN_N, ab);
+		sepd = createState(q);
+		sep2.linkAlphabet(sepd, DIGIT, ab);
+		sep1.linkAlphabet(sepd, DIGIT, ab);
+		sepd.linkAlphabet(sepd, DIGIT, ab);
+		sepd.linkOthers(vertex, e2);
+		sepd.linkEnd(vertex, e2);
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.nina.NinaAction#link(net.morilib.nina.NinaEvent, java.lang.Object)
 	 */
@@ -817,6 +1064,7 @@ public class DFABuilder implements NinaAction {
 		vertex = (ver != null) ? (DBS)ver : createState(q);
 		if((o = q.getEdge()) == null) {
 			v.linkOthers(vertex, q.getLanguageOption().getUnget());
+			v.linkEnd(vertex, null);
 		} else if(o instanceof BuildMachine) {
 			v.linkRecursive(vertex, ((BuildMachine)o).name);
 		} else if(o instanceof RT) {
@@ -835,6 +1083,26 @@ public class DFABuilder implements NinaAction {
 			v.linkDynamic(vertex, ((DY)o).var);
 		} else if(o instanceof RR) {
 			_linkrepr(q, v, ((RR)o).name, ((RR)o).delim);
+		} else if(o instanceof DG) {
+			if(((DG)o).num < 0) {
+				_linkDigitsBig(q, v, ((DG)o).sign);
+			} else {
+				_linkDigitsInt(q, v, ((DG)o).num, ((DG)o).sign);
+			}
+		} else if(o instanceof DH) {
+			if(((DH)o).num < 0) {
+				_linkHexBig(q, v);
+			} else {
+				_linkHexInt(q, v, ((DH)o).num);
+			}
+		} else if(o instanceof DO) {
+			if(((DO)o).num < 0) {
+				_linkOctBig(q, v);
+		} else {
+				_linkOctInt(q, v, ((DO)o).num);
+			}
+		} else if(o == _FLOAT) {
+			_linkFloat(q, v);
 		} else if(o == _END) {
 			v.linkEnd(vertex, mealy);
 		} else {
@@ -1120,4 +1388,33 @@ public class DFABuilder implements NinaAction {
 		return accept(name);
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeDigits(net.morilib.nina.NinaEvent)
+	 */
+	@Override
+	public void setEdgeDigits(NinaEvent q, int digits, boolean sign) {
+		q.setEdge(new DG(digits, sign));
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeHexadecimal(net.morilib.nina.NinaEvent, int)
+	 */
+	@Override
+	public void setEdgeHexadecimal(NinaEvent q, int bits) {
+		q.setEdge(new DH(bits));
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.NinaAction#setEdgeOctal(net.morilib.nina.NinaEvent, int)
+	 */
+	@Override
+	public void setEdgeOctal(NinaEvent q, int bits) {
+		q.setEdge(new DO(bits));
+	}
+
+	@Override
+	public void setEdgeFloat(NinaEvent q) {
+		q.setEdge(_FLOAT);
+	}
+
 }
