@@ -78,4 +78,42 @@ public interface NinaLanguageOption {
 	 */
 	public String getAddList();
 
+	/**
+	 * 
+	 * @return
+	 */
+	public String getSeparator();
+
+	/**
+	 * 
+	 * @return
+	 */
+	public String getSequenceBufferName();
+
+	/**
+	 * 
+	 * @param radix
+	 * @return
+	 */
+	public String getBufferToInteger(int radix);
+
+	/**
+	 * 
+	 * @param radix
+	 * @return
+	 */
+	public String getBufferToBigInteger(int radix);
+
+	/**
+	 * 
+	 * @return
+	 */
+	public String getBufferToNumberInteger();
+
+	/**
+	 * 
+	 * @return
+	 */
+	public String getBufferToNumberDouble();
+
 }
