@@ -15,20 +15,25 @@
  */
 package net.morilib.nina.translate;
 
-import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.PrintStream;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
+import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 import net.morilib.automata.DFAState;
 import net.morilib.nina.DFABuilder;
 import net.morilib.nina.NinaState;
+import net.morilib.nina.Quadro;
+import net.morilib.nina.TMBuilder;
 import net.morilib.nina.translate.sh.ReplaceStrangeChar;
 import net.morilib.range.Interval;
+import net.morilib.sh.ShEnvironment;
 
 /**
  *
@@ -42,6 +47,10 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 			"return +([A-Za-z][A-Za-z0-9]*\\.)*[A-Z][A-Z0-9]*;");
 	private static final Pattern E_CONST = Pattern.compile(
 			"([A-Za-z][A-Za-z0-9]*\\.)*[A-Z][A-Z0-9]*");
+	private static final Pattern _IAT = Pattern.compile(
+			"(.*),([A-Z]+|\\*)");
+	private static final Pattern _CLS = Pattern.compile("<(.*)>");
+	private static final Pattern _STR = Pattern.compile("\"(.*)\"");
 	private static final String LOK = "LOOKAHEAD($c);";
 
 	/**
@@ -79,6 +88,11 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		return x;
 	}
 
+	//
+	private boolean isLookahead(Object o) {
+		return o != null && o.toString().startsWith(LOK);
+	}
+
 	//
 	private void printintv(PrintStream out, Interval v, boolean els,
 			Object o) {
@@ -92,8 +106,7 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		d = ((Integer)v.getSupremumBound()).intValue();
 		x = tostr(c);
 		y = tostr(d);
-		z = o != null && o.toString().startsWith(LOK) ?
-				"__l__ && " : "";
+		z = isLookahead(o) ? "__l__ && " : "";
 		if(v.isClosed() && c == d) {
 			out.format("%s(%s$c == %s) {\n", a, z, x);
 		} else {
@@ -108,63 +121,116 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		String a, z;
 
 		a = els ? "\t\t\t} else if" : "\t\t\tif";
-		z = m != null && m.toString().startsWith(LOK) ?
-				"__l__ && " : "";
+		z = isLookahead(m) ? "__l__ && " : "";
 		out.format("%s(%s$c.equals(\"%s\")) {\n", a, z, o.toString());
 	}
 
 	//
-	private void printclass(PrintStream out, Object o, boolean els,
-			Object m) {
-		String a, s, z;
-		char c;
+	private void printclchar(PrintStream out, char c, String a,
+			String z) {
+		String s;
 
-		a = els ? "\t\t\t} else if" : "\t\t\tif";
-		z = m != null && m.toString().startsWith(LOK) ?
-				"__l__ && " : "";
-		if(o instanceof Character) {
-			if((c = ((Character)o).charValue()) == '\\' || c == '\'') {
-				s = String.format("Character.valueOf('\\%c')", c);
-			} else {
-				s = String.format("Character.valueOf('%c')", c);
-			}
-			out.format("%s(%s%s.equals($c)) {\n", a, z, s);
-		} else if(E_CONST.matcher(s = o.toString()).matches()) {
-			out.format("%s(%s%s.equals($c)) {\n", a, z, s);
+		if(c == '\\' || c == '\'') {
+			s = String.format("Character.valueOf('\\%c')", c);
+		} else if(c < 0x20) {
+			s = String.format("Character.valueOf((char)%d)", (int)c);
 		} else {
-			out.format("%s(%s$c instanceof %s) {\n", a, z, s);
+			s = String.format("Character.valueOf('%c')", c);
 		}
+		out.format("%s(%s%s.equals($c)) {\n", a, z, s);
 	}
 
 	//
-	private String outln(boolean els, PrintStream out) {
-		if(els) {
-			out.println("\t\t\t} else {");
-			return "\t";
+	private boolean printclass1(PrintStream out, Object o, boolean els,
+			Object m) {
+		String a, s, z;
+		Matcher t;
+
+		a = els ? "\t\t\t} else if" : "\t\t\tif";
+		z = isLookahead(m) ? "__l__ && " : "";
+		if(o instanceof Character) {
+			printclchar(out, ((Character)o).charValue(), a, z);
+		} else if(o instanceof Integer) {
+			printclchar(out, (char)((Integer)o).intValue(), a, z);
+		} else if(E_CONST.matcher(s = o.toString()).matches()) {
+			out.format("%s(%s%s.equals($c)) {\n", a, z, s);
+		} else if((t = _STR.matcher(s)).matches()) {
+			out.format("%s(%s\"%s\".equals($c)) {\n", a, z,
+					escapeprint(t.group(1)));
+		} else if((t = _CLS.matcher(s)).matches()) {
+			// do later
+			return false;
+		} else if(s.length() == 1) {
+			printclchar(out, s.charAt(0), a, z);
+			out.format("\t\t\t} else if(%s\"%s\".equals($c)) {\n", z,
+					escapeprint(s));
 		} else {
-			return "";
+			out.format("%s(%s\"%s\".equals($c)) {\n", a, z,
+					escapeprint(s));
 		}
+		return true;
+	}
+
+	//
+	private boolean printclass2(PrintStream out, Object o, boolean els,
+			Object m) {
+		String a, s, z;
+		Matcher t;
+
+		a = els ? "\t\t\t} else if" : "\t\t\tif";
+		z = isLookahead(m) ? "__l__ && " : "";
+		if(o instanceof Character) {
+			// done former
+		} else if(o instanceof Integer) {
+			// done former
+		} else if(E_CONST.matcher(s = o.toString()).matches()) {
+			// done former
+		} else if((t = _STR.matcher(s)).matches()) {
+			// done former
+		} else if((t = _CLS.matcher(s)).matches()) {
+			out.format("%s(%s$c instanceof %s) {\n", a, z, t.group(1));
+			return true;
+		} else if(s.length() == 1) {
+			// done former
+		} else {
+			// done former
+		}
+		return false;
+	}
+
+	//
+	private String outln(boolean els, PrintStream out, String s) {
+		if(els) {
+			out.printf("\t\t\t} else if(%s) {\n", s);
+		} else {
+			out.printf("\t\t\tif(%s) {\n", s);
+		}
+		return "\t";
 	}
 
 	private boolean putend(String s, PrintStream out,
 			DFAState<Object, ?, Void> dfa, boolean els) {
 		DFABuilder.DBS b;
+		TMBuilder.DBS x;
 		Object o;
 
 		if(dfa instanceof DFABuilder.DBS &&
 				(b = ((DFABuilder.DBS)dfa).getEnd()) != null) {
-			if(els) {
-				out.printf("\t\t\t} else if(%s) {\n", s);
-			} else {
-				out.printf("\t\t\tif(%s) {\n", s);
-			}
-
+			outln(els, out, s);
 			if((o = ((DFABuilder.DBS)dfa).getMealyEnd()) != null) {
 				out.format("\t\t\t\t%s\n", o.toString());
 			}
 			out.format("\t\t\t\tSTATE = %d;\n", getStateNo(b));
 			out.format("\t\t\t\treturn 1;\n");
 			els = true;
+		} else if(dfa instanceof TMBuilder.DBS &&
+				(x = ((TMBuilder.DBS)dfa).getEnd()) != null) {
+			outln(els, out, s);
+			printIntervalAction(out, dfa,
+					((TMBuilder.DBS)dfa).getMealyEnd());
+			out.format("\t\t\t\tSTATE = %d;\n", getStateNo(x));
+			out.format("\t\t\t\treturn 1;\n");
+			els = true;
 		}
 		return els;
 	}
@@ -181,9 +247,9 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 				b = ((DFABuilder.DBS)dfa).getUserEdge(x);
 				o = ((DFABuilder.DBS)dfa).getUserMealyEdge(x);
 				if(els) {
-					out.printf("\t\t\t} else if(__g && (%s)) {\n", x);
+					out.printf("\t\t\t} else if(%s) {\n", x);
 				} else {
-					out.printf("\t\t\tif(__g && (%s)) {\n", x);
+					out.printf("\t\t\tif(%s) {\n", x);
 				}
 
 				if(o != null) {
@@ -198,119 +264,302 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 	}
 
 	private boolean putothers(PrintStream out,
-			DFAState<Object, ?, Void> dfa, boolean els) {
-		boolean el2 = true;
+			DFAState<Object, ?, Void> dfa, boolean els, String so) {
 		DFABuilder.DBS a, b;
+		TMBuilder.DBS x;
 		String s, t;
 		Object o;
 
-		if(!(dfa instanceof DFABuilder.DBS)) {
-			// do nothing
+		if(dfa instanceof TMBuilder.DBS) {
+			if((x = ((TMBuilder.DBS)dfa).getOthers()) != null) {
+				s = outln(els, out, so);
+				printIntervalAction(out, dfa,
+						((TMBuilder.DBS)dfa).getMealyOthers());
+				out.format("%s\t\t\tSTATE = %d;\n", s, getStateNo(x));
+				out.format("%s\t\t\treturn 1;\n", s);
+				return true;
+			} else {
+				return els;
+			}
+		} else if(!(dfa instanceof DFABuilder.DBS)) {
+			return els;
 		} else if((b = (a = (DFABuilder.DBS)dfa).getOthers()) != null) {
-			s = outln(els, out);
+			s = outln(els, out, so);
 			if((o = a.getMealyOthers()) != null) {
 				out.format("%s\t\t\t%s\n", s, o.toString());
 			}
 			out.format("%s\t\t\tSTATE = %d;\n", s, getStateNo(b));
 			out.format("%s\t\t\treturn 1;\n", s);
-			el2 = false;
-		} else if((b = a.getRecursive()) != null) {
-			s = outln(els, out);
+			return true;
+		} else if((b = a.getRecursive()) == null) {
+			return els;
+		} else if(a.getRecursiveName().startsWith("g:")) {
+			s = outln(els, out, so);
+			t = a.getRecursiveName().substring(2);
+			out.format("%s\t\t\t__stkpush(%d, new LR%sEngine());\n", s,
+					getStateNo(b), t);
+			out.format("%s\t\t\tSTATE = 0;\n", s);
+			out.format("%s\t\t\treturn NINA_ACCEPT;\n", s);
+			return true;
+		} else {
+			s = outln(els, out, so);
 			t = ReplaceStrangeChar.replace(a.getRecursiveName());
 			out.format("%s\t\t\t__stkpush(%d, ENGINE_%s);\n", s,
 					getStateNo(b), t);
 			out.format("%s\t\t\tSTATE = 0;\n", s);
-			out.format("%s\t\t\treturn -1;\n", s);
-			el2 = false;
+			out.format("%s\t\t\treturn NINA_ACCEPT;\n", s);
+			return true;
+		}
+	}
+
+	//
+	private void printNextStep(PrintStream out,
+			DFAState<Object, ?, Void> d) {
+		out.format("\t\t\t\tSTATE = %d;\n", getStateNo(d));
+		out.println("\t\t\t\treturn 1;");
+	}
+
+	//
+	private void checkRange(Map<Interval, Object> r, Interval v,
+			Object m) {
+		for(Map.Entry<Interval, Object> t : r.entrySet()) {
+			if(!t.getKey().independentOf(v) &&
+					isLookahead(t.getValue()) == isLookahead(m)) {
+				options.pwarn("nondeterministic", v.toString());
+			}
+		}
+		r.put(v, m);
+	}
+
+	//
+	private void checkSet(Map<Object, Object> r, Object v,
+			Object m) {
+		for(Map.Entry<Object, Object> t : r.entrySet()) {
+			if(t.getKey().equals(v) &&
+					isLookahead(t.getValue()) == isLookahead(m)) {
+				options.pwarn("nondeterministic", v.toString());
+			}
+		}
+		r.put(v, m);
+	}
+
+	//
+	private boolean printDynamicTransition(PrintStream out,
+			DFAState<Object, ?, Void> dfa, boolean els) {
+		DFAState<Object, ?, Void> d;
+		String s;
+
+		if(dfa instanceof DFABuilder.DBS &&
+				(d = ((DFABuilder.DBS)dfa).getDynamic()) != null) {
+			s = ((DFABuilder.DBS)dfa).getDynamicVariable();
+			out.print("\t\t\t");
+			if(els)  out.print("} else ");
+			out.printf("if(__l__ && __startWith(%s, $c)) {\n", s);
+			out.format("\t\t\t\t__stkpush(%d, new DynamicEngine(%s, %d));\n",
+					getStateNo(d), s, getStateNo(dfa));
+			out.format("\t\t\t\tSTATE = 0;\n", s);
+			out.format("\t\t\t\treturn NINA_ACCEPT;\n", s);
+			return true;
+		} else {
+			return els;
+		}
+	}
+
+	//
+	private void printWritetape(PrintStream out, String a) {
+		if(a.length() == 0) {
+			// do nothing
+		} else if(a.length() > 1) {
+			out.printf("\t\t\t\t_writetape(\"%s\");\n", a);
+		} else if(a.charAt(0) == '\\' || a.charAt(0) == '\'') {
+			out.printf("\t\t\t\t_writetape('\\%c');\n",
+					a.charAt(0));
+		} else {
+			out.printf("\t\t\t\t_writetape('%c');\n",
+					a.charAt(0));
+		}
+	}
+
+	//
+	private void printIntervalAction(PrintStream out,
+			DFAState<Object, ?, Void> d, Object o) {
+		Matcher m;
+		String s;
+
+		if(d instanceof TMBuilder.DBS) {
+			if(o == null) {
+				out.printf("\t\t\t\t_movetape(R);\n");
+			} else if((s = o.toString()).equals("UNGET($c);")) {
+				// do nothing
+			} else if((m = _IAT.matcher(s)).matches()) {
+				printWritetape(out, m.group(1));
+				if(!m.group(2).equals("*")) {
+					out.printf("\t\t\t\t_movetape(%s);\n", m.group(2));
+				}
+			} else {
+				printWritetape(out, s);
+			}
+		} else if(o != null) {
+			out.format("\t\t\t\t%s\n", o.toString());
 		}
-		return el2;
 	}
 
 	//
 	private void printState(PrintStream out,
 			DFAState<Object, ?, Void> dfa) {
-		boolean els = false, el2 = true;
-		DFAState<Object, ?, Void> d;
+		Map<Interval, Object> r = new HashMap<Interval, Object>();
 		int sn = getStateNo(dfa), c;
+		boolean els = false;
 		Object o;
 
 		out.format("\t\tcase %d:\n", sn);
+
+		// with lookahead
 		for(Interval v : dfa.getAlphabetRanges()) {
 			if(v.isEmpty())  continue;
 			c = infimumbound(v);
 			if(v.isInfimumOpen())  c++;
-			printintv(out, v, els, o = dfa.getLabelInt(c));
-			if(o != null) {
-				out.format("\t\t\t\t%s\n", o.toString());
-			}
+			if(!isLookahead(o = dfa.getLabelInt(c)))  continue;
 
-			// print next step
-			d = dfa.goInt(c);
-			out.format("\t\t\t\tSTATE = %d;\n", getStateNo(d));
-			out.println("\t\t\t\treturn 1;");
+			checkRange(r, v, o);
+			printintv(out, v, els, o);
+			printIntervalAction(out, dfa, o);
+			printNextStep(out, dfa.goInt(c));
 			els = true;
 		}
+
+		// without lookahead
+		for(Interval v : dfa.getAlphabetRanges()) {
+			if(v.isEmpty())  continue;
+			c = infimumbound(v);
+			if(v.isInfimumOpen())  c++;
+			if(isLookahead(o = dfa.getLabelInt(c)))  continue;
+
+			checkRange(r, v, o);
+			printintv(out, v, els, o = dfa.getLabelInt(c));
+			printIntervalAction(out, dfa, o);
+			printNextStep(out, dfa.goInt(c));
+			els = true;
+		}
+
+		// dynamic automaton
+		els = printDynamicTransition(out, dfa, els);
+
 		els = putend("$c < 0", out, dfa, els);
 		els = putuseredges(out, dfa, els);
-		el2 = putothers(out, dfa, els);
+		els = putothers(out, dfa, els, "$c >= 0");
 		if(els)  out.println("\t\t\t}");
-		if(el2)  out.println("\t\t\treturn 0;");
+		out.println("\t\t\treturn 0;");
 	}
 
 	//
 	private void printObjectState(PrintStream out,
 			DFAState<Object, ?, Void> dfa) {
-		boolean els = false, el2 = true;
-		DFAState<Object, ?, Void> d;
+		Map<Object, Object> r = new HashMap<Object, Object>();
 		int sn = getStateNo(dfa);
+		boolean els = false;
 		Object o;
 
 		out.format("\t\tcase %d:\n", sn);
-		for(Object p : dfa.getAlphabets()) {
-			printobj(out, p, els, o = dfa.getLabel(p));
-			if(o != null) {
-				out.format("\t\t\t\t%s\n", o.toString());
-			}
 
-			// print next step
-			d = dfa.go(p);
-			out.format("\t\t\t\tSTATE = %d;\n", getStateNo(d));
-			out.println("\t\t\t\treturn 1;");
+		// with lookahead
+		for(Object p : dfa.getAlphabets()) {
+			if(!isLookahead(o = dfa.getLabel(p)))  continue;
+
+			checkSet(r, p, o);
+			printobj(out, p, els, o);
+			printIntervalAction(out, dfa, o);
+			printNextStep(out, dfa.go(p));
 			els = true;
 		}
+
+		// without lookahead
+		for(Object p : dfa.getAlphabets()) {
+			if(isLookahead(o = dfa.getLabel(p)))  continue;
+
+			checkSet(r, p, o);
+			printobj(out, p, els, o = dfa.getLabel(p));
+			printIntervalAction(out, dfa, o);
+			printNextStep(out, dfa.go(p));
+			els = true;
+		}
+
+		// dynamic automaton
+		els = printDynamicTransition(out, dfa, els);
+
 		els = putend("$c == null", out, dfa, els);
 		els = putuseredges(out, dfa, els);
-		el2 = putothers(out, dfa, els);
+		els = putothers(out, dfa, els, "$c != null");
 		if(els)  out.println("\t\t\t}");
-		if(el2)  out.println("\t\t\treturn 0;");
+		out.println("\t\t\treturn 0;");
 	}
 
 	//
 	private void printClassState(PrintStream out,
 			DFAState<Object, ?, Void> dfa) {
-		boolean els = false, el2 = true;
-		DFAState<Object, ?, Void> d;
+		Map<Object, Object> r = new HashMap<Object, Object>();
 		int sn = getStateNo(dfa);
+		boolean els = false;
 		Object o;
 
 		out.format("\t\tcase %d:\n", sn);
-		for(Object p : dfa.getAlphabets()) {
-			printclass(out, p, els, o = dfa.getLabel(p));
-			if(o != null) {
-				out.format("\t\t\t\t%s\n", o.toString());
-			}
 
-			// print next step
-			d = dfa.go(p);
-			out.format("\t\t\t\tSTATE = %d;\n", getStateNo(d));
-			out.println("\t\t\t\treturn 1;");
+		// with lookahead pass 1
+		for(Object p : dfa.getAlphabets()) {
+			if(!isLookahead(o = dfa.getLabel(p)))  continue;
+			if(!printclass1(out, p, els, o = dfa.getLabel(p))) {
+				continue;
+			}
+			checkSet(r, p, o);
+			printIntervalAction(out, dfa, o);
+			printNextStep(out, dfa.go(p));
 			els = true;
 		}
+
+		// with lookahead pass 2
+		for(Object p : dfa.getAlphabets()) {
+			if(!isLookahead(o = dfa.getLabel(p)))  continue;
+			if(!printclass2(out, p, els, o = dfa.getLabel(p))) {
+				continue;
+			}
+			checkSet(r, p, o);
+			printIntervalAction(out, dfa, o);
+			printNextStep(out, dfa.go(p));
+			els = true;
+		}
+
+		// without lookahead pass 1
+		for(Object p : dfa.getAlphabets()) {
+			if(isLookahead(o = dfa.getLabel(p)))  continue;
+			if(!printclass1(out, p, els, o = dfa.getLabel(p))) {
+				continue;
+			}
+			checkSet(r, p, o);
+			printIntervalAction(out, dfa, o);
+			printNextStep(out, dfa.go(p));
+			els = true;
+		}
+
+		// without lookahead pass 2
+		for(Object p : dfa.getAlphabets()) {
+			if(isLookahead(o = dfa.getLabel(p)))  continue;
+			if(!printclass2(out, p, els, o = dfa.getLabel(p))) {
+				continue;
+			}
+			checkSet(r, p, o);
+			printIntervalAction(out, dfa, o);
+			printNextStep(out, dfa.go(p));
+			els = true;
+		}
+
+		// dynamic automaton
+		els = printDynamicTransition(out, dfa, els);
+
 		els = putend("$c == null", out, dfa, els);
 		els = putuseredges(out, dfa, els);
-		el2 = putothers(out, dfa, els);
+		els = putothers(out, dfa, els, "$c != null");
 		if(els)  out.println("\t\t\t}");
-		if(el2)  out.println("\t\t\treturn 0;");
+		out.println("\t\t\treturn 0;");
 	}
 
 	//
@@ -436,7 +685,7 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 
 		for(DFAState<Object, ?, Void> s : stateKeys()) {
 			if(!s.isAccepted())  continue;
-			p = null;
+			p = null;  m = -1;
 			out.format("\t\tcase %d:\n", stateNo(s));
 			for(Object a : s.getAccepted()) {
 				if(a == null || !(a instanceof NinaState)) {
@@ -494,7 +743,10 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 			} else {
 				p = ReplaceAction.replace(p, a, this, stateNo(s),
 						builder.getLabelByState(s));
-				out.format("\t\t\t%s\n", p);
+				for(String v : p.split("\n")) {
+					out.format("\t\t\t%s\n", v);
+				}
+
 				if(!a[0]) {
 					out.println("\t\t\tbreak;");
 				}
@@ -616,6 +868,17 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		// do nothing
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#printConstants(java.io.PrintStream)
+	 */
+	@Override
+	protected void printConstants(PrintStream out) {
+		for(Map.Entry<String, Integer> t : quadro.getConstantMap().entrySet()) {
+			out.printf("\t\tpublic static final int %s = %d;\n",
+					t.getKey(), t.getValue().intValue());
+		}
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.nina.translate.AbstractNinaTranslator#reportStatistics(java.io.PrintStream)
 	 */
@@ -642,8 +905,8 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 	 */
 	@Override
 	protected PrintStream openOutput() throws IOException {
-		return new PrintStream(new FileOutputStream(
-				getOptions().getJavaFile()), true);
+		return new PrintStream(getOptions().getOutputStream(
+				getOptions().getTitleFile(".java")), true);
 	}
 
 	/* (non-Javadoc)
@@ -743,6 +1006,9 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		ot.append("$$");
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#getConstName(java.lang.String)
+	 */
 	@Override
 	protected String getConstName(String t) {
 		if(t.equals("EXIT")) {
@@ -758,10 +1024,29 @@ public class NinaTranslatorJava extends AbstractNinaTranslator {
 		}
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#appendYield(java.lang.StringBuffer, java.lang.String)
+	 */
 	@Override
 	protected void appendYield(StringBuffer ot, String b) {
 		ot.append("yieldObject = (").append(b).append(");");
 		ot.append("return NINA_YIELD;");
 	}
 
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#getLRTranslator(net.morilib.nina.Quadro)
+	 */
+	@Override
+	protected LRTranslator getLRTranslator(Quadro q) {
+		return new LRTranslatorJava(q);
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.nina.translate.AbstractNinaTranslator#setLocalenv(net.morilib.sh.ShEnvironment)
+	 */
+	@Override
+	protected void setLocalenv(Quadro q, ShEnvironment env) {
+		env.bind("VALTYPE", q.getOptionNvl("valtype", "Object"));
+	}
+
 }
