@@ -640,7 +640,10 @@ public class NinaTranslatorJavaScript extends AbstractNinaTranslator {
 			} else {
 				p = ReplaceAction.replace(p, a, this, stateNo(s),
 						builder.getLabelByState(s));
-				out.format("\t\t%s\n", p);
+				for(String v : p.split("\n")) {
+					out.format("\t\t%s\n", v);
+				}
+
 				if(!a[0]) {
 					out.println("\t\tbreak;");
 				}
